// ====================================================================
// Utility class to store a range and increment when used.  Several
// can be chained together, such that when when one counter hits the
// end of its range and wraps, the next most significant one can be
// triggered to increment.  An set of autorange objects that can 
// represent 00 to 29 would be  0-2 <- 0-9 <- ptr.   A set of 
// objects that can represent a00 to c39 would be 
// a-c <- 0-3 <- 0-9 <- ptr.
// ====================================================================

using namespace std;

#include <iostream>
#include <iomanip>
#include <string>
#include <vector>

#include "autorange.h"
#include "psstream.h"
#include "utility.h"

// Increment cur (possibly by zero) and return the previous value of
// curstr.

string autorange::incr(int by)  // Return curstr, postincr cur
{  
   string s;
   // Increment leftward digits if we're wrapping,
   // get leftward digits in string.
   if (left) s = left->incr((cur+by > end) ? 1 : 0);
   s += curstr;
   cur += by;
   if (cur > end) cur = start; 
   if (by > 0) setcurstr();
   return s;
};

// How many objects can this autorange enumerate?

unsigned int autorange::count(void)				
{  
   unsigned int ct = 0;					
   if (left) ct = left->count();
   return ct*(end-start+1);
};

// Constructor for autonum object

autonum::autonum(string s, class autorange *l) 
{  
   vector<string> p = Tokenize(s, "-\t\n\r "); 
   cur = start = StrToInteger(p[0]); 
   end = StrToInteger(p[1]);
   width = p[0].size();
   left = l;
   setcurstr();
};

// Variant of setcurstr for numeric objects.

void autonum::setcurstr(void)
{  
   OSTRSTREAM s;
   s << setw(width) << setfill('0') << cur;
   curstr = s.str();
};

// Constructor for autostr object

autostr::autostr(string s, autorange *l) 
{  
   vector<string> p = Tokenize(s, "-"); 
   cur = start = p[0][0];
   end = p[1][0];
   width = p[0].size();
   left = l;
   setcurstr();
};

// Variant of setcurstr for char objects

void autostr::setcurstr(void) 
{ 
   curstr = " "; 
   curstr[0] = cur; 
};

// Constructor for autoconst object

autoconst::autoconst(string s, autorange *l) 
{  
   cur = start = 0;
   end = 32767;
   width = s.size();
   left = l;
   curstr = s;
};

// Variant of setcurstr for const objects

void autoconst::setcurstr(void) 
{ 
};

// Pass any increment on to the left.

string autoconst::incr(int by)
{  
   string s;
   if (left) s = left->incr(by);
   s += curstr;
   return s;
};

unsigned int autoconst::count(void)
{  
   if (!left) return 1;
   return left->count();
};
