// ====================================================================
// Utility class to store a range and increment when used.  Several
// can be chained together, such that when when one counter hits the
// end of its range and wraps, the next most significant one can be
// triggered to increment.  An set of autorange objects that can 
// represent 00 to 29 would be  0-2 <- 0-9 <- ptr.   A set of 
// objects that can represent a00 to c39 would be 
// a-c <- 0-3 <- 0-9 <- ptr.
// ====================================================================

#include <string>

// First declare the virtual base class.

class autorange
{
protected:
   int cur;                     // Current value of this counter
   int start;                   // Value to start counting from
   int end;                     // End value
   int width;                   // Width of text string when output
   string curstr;               // String representation of cur
   class autorange *left;       // More significant counter
  public:
   autorange(void)              // Constructor for default object.
      : cur(0), start(0), end(0), width(0), left(NULL) {	};
   
   virtual ~autorange(void)     // Object destructor
      { if (left) delete left; left = NULL; };
   virtual void setcurstr(void) = 0; // Convert cur to string
   virtual string incr(int by = 1);  // Return curstr, postincr cur
   virtual unsigned int count(void); // How many objects can this enumerate?
};

// Now a subclass to do ranges of digits given a string
// specifying starting and ending values.  The values
// are padded with zeros to the width of the first value,
// so 0-20 would generate 1,2,3...20, while 00-20 would
// generate 01,02,03...20

class autonum : public autorange
{
public:
   autonum(string s, class autorange *l = NULL);
   ~autonum(void) {  };
   void setcurstr(void);
};

// Subclass to do the same thing with single character ranges.
// (subsequent characters silently ignored)

class autostr : public autorange
{
public:
   autostr(string s = "", autorange *l = NULL);
   ~autostr(void) { };
   void setcurstr(void);
};

// Subclass for the degenerate non-range constant token.

class autoconst : public autorange
{
public:
   autoconst(string s, autorange *l = NULL);
   void setcurstr(void);
   string incr(int by = 1);
   unsigned int count(void);
};
