// ====================================================================
// General class to parse command line options.  Accepts both single
// character and extended options, with abbreviations.
// ====================================================================

// ******************************************************************** //
//                  SLAC Console Management Tools                       //
//         Chuck Boeheim, Stanford Linear Accelerator Center            //
//                                                                      //
// $Revision: 1.2 $ $Date: 2005/03/10 15:32:50 $ $Author: boeheim $
// ******************************************************************** //

#ifndef cmdargs_h
#define cmdargs_h

#include <string>
#include <vector>
#include <utility>

typedef pair<int,string> argpair;

// Item to be thrown on case of error, containing the reason for the
// error and the offending option.

class CmdErr
{
public:
   string error;
   string option;
   
   CmdErr(string e, string o) : error(e), option(o) {};
};

// Single option description.

class OptionDesc
{
   int handle;
   string pattern;
   bool single;
   bool takesvalue;
   int minlength;
public:
   OptionDesc(int h, string p, bool tv, int min);
   
   int  Handle(void) { return handle; };
   bool Matches(string token);
   bool SingleChar(void) { return single; };
   bool TakesValue(void) { return takesvalue; };
};

class CmdArgs
{
private:
   vector<OptionDesc*> desclist;
   vector<argpair> optlist;
   vector<string> arglist;
public:
   ~CmdArgs(void);
   
   void   AddOption(int h, string p, bool tv, int min=1);
   void   ParseArgs(int, char *[]) throw(CmdErr);
   int    OptCount (void)    { return optlist.size(); };
   int    Option   (int ind) { return optlist[ind].first; };
   string Value    (int ind) { return optlist[ind].second; };
   int    ArgCount (void)    { return arglist.size(); };
   string Arg      (int ind) { return arglist[ind];   };
};

#endif
