// ====================================================================
// consolec.hh -- Class data for the console client main routine.
// ====================================================================

// ******************************************************************** //
//                  SLAC Console Management Tools                       //
//         Chuck Boeheim, Stanford Linear Accelerator Center            //
//                                                                      //
// $Revision: 1.7 $ $Date: 2007/04/16 01:59:11 $ $Author: boeheim $
// ******************************************************************** //

#ifndef consolec_hh
#define consolec_hh

#include <string>
#include <vector>

#include "cmdargs.h"
#include "host.h"
#include "syslog.h"
#include "terminal.h"
#include "server.h"

typedef vector<Host> Hostlist;

class funcobj; // Class to be defined in the implementaion.

class ConsoleClient
{
private:
   CmdArgs     myargs;          // Program arguments;
   string      username;        // Our user name.
   string      pass;            // User password.
   string      gssservice;      // GSSAPI service principal
   string      version;         // Version string
   
   vector<string> configs;      // List of config files to check.

   int         port;            // Default port for connections
   Hostlist    masterlist;      // List of available master servers.
	
   string      console;         // Console name requested.
   string      response;        // Response from command.

   Server      server;          // The connection to the server.
   Terminal    terminal;        // The connection to the terminal.

   File       *StdOut;          // What stdout is connected to.

   funcobj    *function;        // Function to execute for request.

   void   UsageShort(void);
   void   UsageLong(void);
   Hostlist MakeHostlist(const string &s); // Make a hostlist from a string.
   Hostlist MakeHostlist(vector<string> &s); // Make a hostlist from tokens.

public:
   ConsoleClient(void);
   
   string Username(void) { return username; };

   void   ParseArgs(int argc, char *argv[]);
   void   ProcessArgs(void);
   void   ReadConfig(string filename);
   void   ReadConfigs(void);

   void   Run(void);
   string Distribute(const string &target, const string &cmd);
   string Cmd(Hostlist &hl, const string &cmd);
   string Connect(const string &target);
};

extern ConsoleClient CC;

#endif
