// ====================================================================
// dispatcher.hh - Object to own the console lines.
// ====================================================================

// ******************************************************************** //
//                  SLAC Console Management Tools                       //
//         Chuck Boeheim, Stanford Linear Accelerator Center            //
//                                                                      //
// $Revision: 1.4 $ $Date: 2006/08/21 23:59:14 $ $Author: boeheim $
// ******************************************************************** //

#ifndef dispatcher_h
#define dispatcher_h

#include "client.h"
#include "console.h"
#include "slock.h"
#include "sockettcp.h"

#include <list>
#include <pthread.h>

#ifdef HAVE_LIBSSL
  #include "socketssl.h"
  #define SOCK_TYPE SocketSSL
#else
  #define SOCK_TYPE SocketTCP
#endif

class Dispatcher
{
public:
   typedef list<Console *> consolelist;
   typedef list<BConsole *> rconsolelist;
   typedef list<Client *> clientlist;
   
private:
   consolelist  consoles;       // List of all local consoles
   rconsolelist rconsoles;      // List of all remote consoles.
   clientlist   clients;        // List of all connected clients

   MLock consoleLock;           // Lock access to local consoles.
   MLock rconsoleLock;          // Lock access to remote consoles.
   MLock clientLock;            // Lock access to clients.
   
   SOCK_TYPE listenSocket;      // Socket to listen for connections.
   pthread_t listenThread;      // ID of thread for listening.
   SLock modLock;               // Lock dispatcher while reading lists.

   SLock halt;                  // Condition for halting.

public:
   Dispatcher(void) {};
   
   void Run(void);              // Listen waiting for connections.
   void WaitForConnection(void); // Method for listening thread to run.
   void Stop(string reason);

   void StartConsoles(void);     // Start console threads.
   void RegisterConsole(Line *); // Add/update a console.
   Console *FindConsole(string); // Search for a console by name.
   BConsole *FindRConsole(string);

   consolelist::iterator FirstConsole(void) { return consoles.begin(); };
   consolelist::iterator LastConsole(void)  { return consoles.end(); };
   clientlist::iterator  FirstClient(void)  { return clients.begin(); };
   clientlist::iterator  LastClient(void)   { return clients.end(); };

   void LockClientlist   (MLock::locktype how) { clientLock.Lock(how); };
   void UnlockClientlist (MLock::locktype how) { clientLock.Unlock(how); };
   void LockConsolelist  (MLock::locktype how) { consoleLock.Lock(how); };
   void UnlockConsolelist(MLock::locktype how) { consoleLock.Unlock(how); };

   void Check(void);            // Check lists for consistancy
};

#endif
