// ====================================================================
// Class for encapsulating files and their actions.
// ====================================================================

// ******************************************************************** //
//                  SLAC Console Management Tools                       //
//         Chuck Boeheim, Stanford Linear Accelerator Center            //
//                                                                      //
// $Revision: 1.1 $ $Date: 2005/02/09 00:38:49 $ $Author: boeheim $
// ******************************************************************** //

#ifndef file_hh
#define file_hh

#include <string>

#include <fcntl.h>
#include <sys/stat.h>
#include <sys/ioctl.h>

class Buffer;

class File
{
protected:

   int fd;                      // File descriptor.
   string filename;             // File name.

   Buffer *holding;             // holding place if blocked.

public:

   class IOErr                  // What is thrown on error.
   {
   public:
      const string routine;
      const int    reason;
      const string message;
      const string filename;
      IOErr(const string rt = "",
            const int    rs = 0,
            const string fn = "")
          : routine(rt),
            reason(rs),
            message(strerror(reason)),
            filename(fn) {};
      IOErr(const string rt,
            const int    rs,
            const string fn,
            const string ms)
          : routine(rt),
            reason(rs),
            message(ms),
            filename(fn) {};
   };
   
   File(int f = -1);              // Constructor.
   File(const File &);            // Copy constructor.
   File &operator=(const File &); // Assignment operator.
   ~File(void);                   // Destructor.

   void Open(const string filename, int flags=0, int mode=0666);
   void Dup(File &fp);
   void Close(void);
   void NonBlocking(void);      // Set non-blocking I/O
   void Blocking(void);         // Set blocking I/O
   int  BufferSize(int);        // Set and query buffer size.
   int  BufferSize(void);
   void Write(const Buffer &buffer);
   void Write(const string &buffer, int len=0);
   void Write(const char *buffer, int len=0);
   void Write(const char cc);
   int  Read(Buffer &buffer);
   int  Read(char *buffer, int len);
   int  IOCtl(int flags, void *data = 0);
   void Stat(struct stat&);
   void Chown(int uid, int gid);
   void Chmod(int mode);

   const int FD(void) const { return fd; };
   const string FileName(void) const { return filename; }; 
};
   

#endif
