// ====================================================================
// gssapi.h -- A class for gssapi authentication
// ====================================================================

// ******************************************************************** //
//                  SLAC Console Management Tools                       //
//         Chuck Boeheim, Stanford Linear Accelerator Center            //
//                                                                      //
// $Revision: 1.4 $ $Date: 2006/08/21 23:59:14 $ $Author: boeheim $
// ******************************************************************** //

#ifndef gssapi_hh
#define gssapi_hh

#include "autoconfig.h"
#include "socket.h"

#ifdef HAVE_GSSAPI_H
# include <gssapi.h>
#elif HAVE_GSSAPI_GSSAPI_H
# include <gssapi/gssapi.h>
#endif
#ifdef HAVE_GSSAPI_GSSAPI_GENERIC_H
# include <gssapi/gssapi_generic.h>
#endif

class GSSAPI
{
private:
   Socket          *socket;       // Socket object to communicate with.
   gss_ctx_id_t    gsscontext;    // Pointer to the context.
   gss_cred_id_t   gsscredential; // Pointer to the credentials.
   string          clientname;    // Authenticated client name.
   string          service;       // Service name to authenticate to.
   gss_name_t      gssservice;    // Internal name of service.
   char            gsserror[256]; // last error message

   OM_uint32 majorStatus;       // Return status from most recent operation.
   OM_uint32 minorStatus;       //
   OM_uint32 gssflags;          // Connection flags.

   static gss_cred_id_t gssdefaultcred; // default credential
   
   enum {HANDSHAKE='H', DATA='D', PROTOCOL_ERR='E' };
   void SendPacket(char type, gss_buffer_t buffer);
   char ReceivePacket(gss_buffer_t buffer);

public:
   GSSAPI(Socket *s);           // Constructor
   
   ~GSSAPI(void);               // Destructor

   // Establish Client or Server contexts
   bool AcquireServerCreds(const string service_name) // Set service name and get
      { return AcquireServerCreds(service_name.c_str()); };
   bool AcquireServerCreds(const char *service_name); // credentials for it.
   bool SetService(const string service_name)
      { return SetService(service_name.c_str()); };
   bool SetService(const char *service_name);         // Set service name for client to use;
   bool Connect(void);           // Client connect to server.
   bool Accept(void);            // Server accept and authenticate client.

   // Get authenticated client name.
   string ClientName(void) { return clientname; };
   gss_cred_id_t Credential(void) { return gsscredential; };
   
   // Display GSSAPI information.
   bool NotOK(const char *msg);
   void LogMsg(const char *msg, OM_uint32 code1, int code2);
   void DisplayCtxFlags(OM_uint32 flags);
   char *ErrMessage(void) { return gsserror; }
   
   // Decode and display a token. 
   void PrintToken(gss_buffer_t tok);

   // Set default credentials for all subsequent gssapi objects
   static bool DefaultCreds(const string service_name)
      { return DefaultCreds(service_name.c_str()); };
   static bool DefaultCreds(const char *service_name);
};
   

#endif
