// ====================================================================
// heartbeat.cc - listen for a heartbeat from a system.
// ====================================================================

// ******************************************************************** //
//                  SLAC Console Management Tools                       //
//         Chuck Boeheim, Stanford Linear Accelerator Center            //
//                                                                      //
// $Revision: 1.1 $ $Date: 2005/02/09 00:38:49 $ $Author: boeheim $
// ******************************************************************** //

#ifndef heartbeat_hh
#define heartbeat_hh

#include <string>
#include <time.h>

#include "buffer.h"

class Console;

class HeartBeat
{

private:

   Console *console;            // Pointer to containing console.
   int    lastseq;              // Last sequence number recvd.
   time_t lasttime;             // Last time heartbeat heard.
   string host;                 // Last hostname received.
   SmallBuffer logmsg;          // Construct logmsg.
   enum {HB_NONE, HB_OK, HB_STOPPED, HB_SHUTDOWN} 
      state;                    // Current state of detection.
   bool changed;                // Record if state changed.
   
public:

   HeartBeat(Console *);        // Constructor.
   void Input(Buffer &);        // Get input from console
   void IdleCheck(time_t now);  // Called by dispatcher on idle
   void Display(Buffer &);      // Called by console when it
   bool GetState(Buffer &);     // Append state to record.
   void SetState(Buffer &);     // Set state from record.
};

#endif
