// ====================================================================
// Host.hh -- An abstract class for host names and addresses.
// ====================================================================

// ******************************************************************** //
//                  SLAC Console Management Tools                       //
//         Chuck Boeheim, Stanford Linear Accelerator Center            //
//                                                                      //
// $Revision: 1.1 $ $Date: 2005/02/09 00:38:49 $ $Author: boeheim $
// ******************************************************************** //

#ifndef host_hh
#define host_hh

#include <netdb.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <string>

// Use these typedefs to insulate against changes in data 
// representation in calling programs.

typedef unsigned long int addrt;

class Host
{
private:
   struct sockaddr_in sa;       // Internet address structure.
   bool   addrset;              // True if lookup has set address.
   string name;                 // Primary name.
   int error;                   // h_errno value returned from lookup.
   
   static bool initialized;     // Flag and function to initialize 
   static void initialize(void); // the static members.
   
   static string thishostname;  // Local host name.
   static string thisdomainname; // Local domain name.
   static string thisfqname;    // Local fully-qualified name.
   static string thisipaddr;    // Local ip address.

   void SetNameAddr(struct hostent *);

public:
   Host(void);
   bool AddrSet(void) { return addrset; }

   bool Find(string);           // Find host by name
   bool Find(addrt);            // Find host by addr

   bool FindPort(const string&, const string&);   // Find service by name and proto
   bool FindPort(int, const string&); // Find service by name and proto
   void SetPort(short);		// Set specific port number.

   bool GetPeerName(int fd);    // Get the peername of a socket.
   bool GetPeerName(struct sockaddr_in addr);    // Get the peername of an address.
   bool GetPeerName(void);      // Get the peername of the set address.

   short  Port(void);		// Return port number.
   string Name(void);           // Return node name, suppress domain if equal to local.
   string Node(void);           // Return just primary node name.
   string Domain(void);         // Return domain part of node name.
   string FQName(void);         // Return fully-qualified name.
   addrt  Addr(void);           // Return primary addr
   string AddrStr(void);        // Return primary addr in string format.

   static string LocalNode(void);
   static string LocalDomain(void);
   static string LocalFQName(void);
   static string LocalAddr(void);
   
   int Error(void) { return error; };   

   struct sockaddr *Ptr(void) { return (sockaddr *) &sa; }
   int Len(void) { return sizeof(sa); }
};

#endif
