// ====================================================================
// Logfile class.  Log is opened on demand by writing to the file, and
// supports autoclosing.
// ====================================================================

// ******************************************************************** //
//                  SLAC Console Management Tools                       //
//         Chuck Boeheim, Stanford Linear Accelerator Center            //
//                                                                      //
// $Revision: 1.1 $ $Date: 2005/02/09 00:38:50 $ $Author: boeheim $
// ******************************************************************** //

#ifndef logfile_h
#define logfile_h

#include "file.h"
#include "slock.h"
#include "buffer.h"

#include <string>

class Logfile : public File
{
private:
   string  name;                // Name of log file.
   string  dir;                 // Directory to hold log file.
   int     owner;               // User who owns logfile.
   int     ownergrp;            // Group of logfile.
   int     mode;                // Filemode of logfile.
   SLock   logopen;             // True if log is open, lock when changing.

   void OpenLog (bool locklog);
   void CloseLog(bool locklog);

public:
   Logfile(void);
   ~Logfile(void) { CloseLog(); };
   
   string Name    (void) { return name; };
   string Dir     (void) { return dir; };
   int    Owner   (void) { return owner; };
   int    OwnerGrp(void) { return ownergrp; };
   int    Mode    (void) { return mode; };
   string Fullname(void);

   string Name    (string n) { return name = n; };
   string Dir     (string d) { return dir = d; };
   int    Owner   (int o   ) { return owner = o; };
   int    OwnerGrp(int g   ) { return ownergrp = g; };
   int    Mode    (int m   ) { return mode = m; };

   void OpenLog(void);
   void CloseLog(void);
   void WriteLog(Buffer &record);
   typedef enum { Head, Cur, Tail } howval;
   void ReadLog(Buffer &buffer, howval how);
};

#endif
