// ====================================================================
// Class to implement multiple locks.  A lock can be locked for
// read-only access or read-write access.  Any number of threads
// can hold a read-only lock; only one thread can hold a read-write
// lock.  Writers have priority over readers.
// ====================================================================

// ******************************************************************** //
//                  SLAC Console Management Tools                       //
//         Chuck Boeheim, Stanford Linear Accelerator Center            //
//                                                                      //
// $Revision: 1.1 $ $Date: 2005/02/09 00:38:50 $ $Author: boeheim $
// ******************************************************************** //

#ifndef mlock_hh
#define mlock_hh

#include <pthread.h>

class MLock
{
private:

   pthread_mutex_t mutex;       // Mutex to protect the struct.
   pthread_cond_t swait;        // Var enqueues waiting shared.
   pthread_cond_t xwait;        // Var enqueues waiting exclusive.
   int numLocks;                // +n = number of shared locks,
                                // -1 = number of exclusive locks.
   int waitingXLocks;           // Num of exclusives waiting.
   bool predicate;              // Predicate for testing.

   friend void xcleanup(void *); // Internal cleanup functions
   friend void scleanup(void *); // 

public:

   MLock(void);                 // Constructor.
   MLock(MLock &);              // Copy constructor.
   ~MLock(void);                // Destructor.

   typedef enum {Shared, Exclusive} locktype;    // Select type of lock.

   // Provide a Predicate for use with the lock.
   // The predicate is the name of the class itself, i.e.
   // SLock mylock;  mylock = true;   if (mylock) ...
   inline bool operator= (bool p) { return predicate = p; };
   inline      operator bool()    { return predicate; };

   void Lock(locktype type=Shared);     // Obtain the lock.
   bool TryLock(locktype type=Shared);  // Obtain the lock unless locked exclusve
   void Unlock(locktype type=Shared);   // Release the lock.
};

   
#endif
