// ====================================================================
// msgclient.hh -- Interface to Message Service
// ====================================================================

// ******************************************************************** //
//                  SLAC Console Management Tools                       //
//         Chuck Boeheim, Stanford Linear Accelerator Center            //
//                                                                      //
// $Revision: 1.2 $ $Date: 2005/04/07 18:21:22 $ $Author: boeheim $
// ******************************************************************** //

#ifndef msgclient_hh
#define msgclient_hh

#include "buffer.h"
#include "host.h"
#include "sockettcp.h"
#include "socketudp.h"

#include <vector>

typedef enum {UNDEF, SUBSCRIBER='C', SUPPLIER='P'} MessageDirection;
typedef enum {TCP, UDP} ConnectionType;

class MessageClient
{
private:
   ConnectionType type;         // Type of connection.
   Host host;                   // Host & port to connect to.
   string username;             // User requesting service.
   
   SmallBuffer control;         // Formatted control record.
   Socket* server;              // Socket to open to server.
public:
   MessageClient(void) { type = TCP; }
   ~MessageClient(void);        // Destructor
   void SetConnection(ConnectionType t) { type = t; }
   void OpenStream(MessageDirection dir, Host &host, string stream);
   void OpenStream(MessageDirection dir, Host &host, vector<string> &streamlist);
   void ReopenStream(void);
   void CloseStream(void);
   void Send(Buffer &record);
   int  Receive(Buffer &record);
   bool ConnectionIs(ConnectionType t) { return type == t; }
};

#endif
