// ====================================================================
// mydbm.cc - Interface to dbm/gdbm/ndbm.
//            This is not intended to be a full encapsulation, but
//            just the minimum needed by the console package.
//            If no dbm package is available, it becomes a no-op.
// ====================================================================

// ******************************************************************** //
//                  SLAC Console Management Tools                       //
//         Chuck Boeheim, Stanford Linear Accelerator Center            //
//                                                                      //
// $Revision: 1.3 $ $Date: 2007/04/02 16:07:28 $ $Author: boeheim $
// ******************************************************************** //

using namespace std;

#include "autoconfig.h"
#include "buffer.h"
#include "mydbm.h"

#include <string>

#if defined(HAVE_LIBGDBM)
  #include <gdbm.h>
#elif defined(HAVE_NDBM)
  #include <fcntl.h>
  #include <ndbm.h>
#else
   typedef struct datum 
   {
      const char *dptr;
      int  dsize;
   } datum;
#endif

bool MyDBM::Open(void)
{
   if (isopen) return true;
#if defined(HAVE_LIBGDBM)
   dbmfile = gdbm_open(filename, 512, GDBM_WRCREAT, 0755, NULL);
#elif defined(HAVE_LIBNDBM)
   dbmfile = dbm_open(filename, O_RDWR|O_CREAT, 0755);
#endif
   if (dbmfile) isopen = true;
   return isopen;
};

void MyDBM::Store(string key, Buffer &record)
{
   datum keydatum, recorddatum;
   
   if (! isopen) return;
   keydatum.dptr     = (char *) key.c_str();
   keydatum.dsize    = key.size() + 1;
   recorddatum.dptr  = (char *) record.RPos();
   recorddatum.dsize = record.Size();
   
#if defined(HAVE_LIBGDBM)
   gdbm_store((GDBM_FILE) dbmfile, keydatum, recorddatum, GDBM_REPLACE);
#elif defined(HAVE_LIBNDBM)
   dbm_store(dbmfile, keydatum, recorddatum, DBM_REPLACE);
#endif
};

int MyDBM::Fetch(string key, Buffer &record)
{
   datum keydatum, recorddatum;
   
   record.Clear();
   
   if (! isopen) return 0;
   
   keydatum.dptr     = (char *) key.c_str();
   keydatum.dsize    = key.size() + 1;
   
#if defined(HAVE_LIBGDBM)
   recorddatum = gdbm_fetch((GDBM_FILE) dbmfile, keydatum);
#elif defined(HAVE_LIBNDBM)
   recorddatum = dbm_fetch(dbmfile, keydatum);
#endif
   
   if (recorddatum.dptr)
      record.Append(recorddatum.dptr, recorddatum.dsize);
   
   return recorddatum.dsize;
};

void MyDBM::Close(void)
{
   if (! isopen) return;
   
#if defined(HAVE_LIBGDBM)
   gdbm_close((GDBM_FILE) dbmfile);
#elif defined(HAVE_LIBNDBM)
   dbm_close(dbmfile);
#endif
   
   isopen = false;
};


