// ====================================================================
// reader.hh - Virtual class to define behaviors of other classes that
//             can create threads to read from a file.
// ====================================================================

// ******************************************************************** //
//                  SLAC Console Management Tools                       //
//         Chuck Boeheim, Stanford Linear Accelerator Center            //
//                                                                      //
// $Revision: 1.3 $ $Date: 2006/08/21 23:59:14 $ $Author: boeheim $
// ******************************************************************** //

#ifndef reader_hh
#define reader_hh

#include <signal.h>
#include <pthread.h>

class Reader
{
private:
   
   pthread_t thread;            // Posix thread identifier

public:
   
   Reader(void) : thread(0) {};
   virtual ~Reader(void) {};

   virtual void ReaderProc(void) = 0;        // Override to read specific device.

   void StartReader(void);      // Start the reader thread.
   void StopReader(void);       // Cancel the reader thread.
   void Detach(void);           // Detach the reader thread.
   void Join(void);             // Join the reader thread.
   void Exited(void)       { thread = 0; };

   bool Started(void)      { return (thread != 0); };
   pthread_t Thread(void)  { return thread; }
   bool Running(void)      { return (thread != 0) && (pthread_kill(thread, 0) == 0); }; 
};

#endif






