// ====================================================================
// sensorsoft.cc - A filter for the sensorsoft temperature sensors.
// ====================================================================

// ******************************************************************** //
//                  SLAC Console Management Tools                       //
//         Chuck Boeheim, Stanford Linear Accelerator Center            //
//                                                                      //
// $Revision: 1.1 $ $Date: 2005/02/09 00:38:50 $ $Author: boeheim $
// ******************************************************************** //

#ifndef sensorsoft_hh
#define sensorsoft_hh

#include <string>

#include "buffer.h"
#include "filter.h"

#define STATUS "\xc1\x0b\x00\x01\x00\x00\x00\x00\x00\x47\x98"
#define STATLEN  11
#define QID    "\xc3\x0b\x00\x01\x00\x00\x00\x00\x00\x20\x5e"
#define QIDLEN   11
#define QUERY  "\xc5\x0c\x00\x01\x00\x00\x00\x00\x00\x01\x0e\x49"
#define QUERYLEN 12
#define INTVL (60*10)

class SensorSoft : public Filter
{

private:

   int    nextQuery;            // Time at which to read sensor.
   int    state;                // Type of response awaited.
   bool   isOpen;               // Show that console is open.
   SmallBuffer input;           // Buffer of sensor data.
   SmallBuffer logbuffer;       // Buffer for log message.
   string name;                 // Name of sensor.
   string model;                // Model of sensor.
   string firmware;             // Firmware revision.
//   File db;
   
public:

   SensorSoft(Console *);       // Constructor.
   void Input(Buffer &);        // Get input from console
   void IdleCheck(time_t now);  // Called by dispatcher on idle
   void Open(void);             // Called by console when it
   void Close(void);            //   opens and closes.
};

#endif
