// ====================================================================
// server.hh - Class representing the connection to the server 
//             process.  It knows about an associated terminal,
//             and runs a reader thread.
// ====================================================================

// ******************************************************************** //
//                  SLAC Console Management Tools                       //
//         Chuck Boeheim, Stanford Linear Accelerator Center            //
//                                                                      //
// $Revision: 1.3 $ $Date: 2006/12/05 06:44:28 $ $Author: boeheim $
// ******************************************************************** //

#ifndef server_hh
#define server_hh

#include "autoconfig.h"
#include "sockettcp.h"
#include "reader.h"
#include "buffer.h"
#include "file.h"
#include "syslog.h"
#include "slock.h"

#include "terminal.h"

#ifdef HAVE_LIBSSL
  #include "socketssl.h"
  #define SOCK_TYPE SocketSSL
#else
  #define SOCK_TYPE SocketTCP
#endif

class Server : public SOCK_TYPE, public Reader
{
private:
   
   File       *terminal;
   Buffer      termbuffer;
   string      gssservice;      // GSSAPI service name
   
public:

   Server(void);
   ~Server(void);
   
   // Terminal method sets the pointer to the terminal.
   // It's a simple File class instead of a terminal, so
   // that it can be re-directed to stdout.  This class
   // doesn't do anything special to the characteristics
   // of the terminal, though the caller might.

   void   RegisterTerminal(File *term) { terminal = term; }; 
   string GetNextWord(void);
   string GetNextResponse(void);
   void   ReadToEnd(void);
   void   AuthPass(void);
   void   AuthGSSAPI(void);

   void   ReaderProc( void );

   void   SetGSSService(string s) { gssservice = s; };
   
};

#endif

