// ====================================================================
// Socketssl.hh -- A class for SSL TCP sockets.
// ====================================================================

// ******************************************************************** //
//                  SLAC Console Management Tools                       //
//         Chuck Boeheim, Stanford Linear Accelerator Center            //
//                                                                      //
// $Revision: 1.2 $ $Date: 2005/04/07 18:21:24 $ $Author: boeheim $
// ******************************************************************** //

#ifndef socketssl_hh
#define socketssl_hh

#include "sockettcp.h"

#ifndef SSLptr
#define SSLptr void
#endif

class SocketSSL : public SocketTCP
{
private:
   SSLptr *ssl;
   string peerCN;

   void ThrowSSLErr(char *rtn);
   void SSLSetup(void);
   void AcceptSSL();            // SSL portion of Accept.
   
public:
   SocketSSL(void);             // Constructors
   SocketSSL(int fd);
   SocketSSL(SocketTCP &);
   
   ~SocketSSL(void);            // Destructor

   static void SetCAFile(const char *file);
   static void SetCAPath(const char *path);
   static void SetCertFile(const char *file);
   static void SetKeyFile(const char *file);

   Buffer &GetSSLCipher(Buffer &buffer);
   string GetPeerCN(void) { return peerCN; };
   
   void Create(void);
   SocketSSL *Accept(void);     // Accept incoming connections.
   SocketSSL *Accept(SocketSSL *);
   void Connect(Host host);     // Connect to a remote host.
   bool CheckCert(void);        // Check peer certificate.
   
   // Send function overloaded to take different data types.

   void Send(const Buffer &buffer)                   { Send(buffer.RPos(), buffer.Size()); };
   void Send(const string &buffer, int len=0)        { Send((unsigned char *)buffer.c_str(), len); };
   void Send(const unsigned char cc)                 { Send(&cc, 1); };
   void Send(const unsigned char *buffer, int len=0);

   // Receive function overloaded to take different data types.
   int  Receive(Buffer &buffer) 
      { int len = Receive(buffer.WPos(), buffer.Space());
        buffer.WSeek(Buffer::Current, len);
        return len;
      };
   int  Receive(unsigned char *buffer, int len);
};
   

#endif
