// ====================================================================
// Sockettcp.cc -- A class implementing tcp sockets.
// ====================================================================

// ******************************************************************** //
//                  SLAC Console Management Tools                       //
//         Chuck Boeheim, Stanford Linear Accelerator Center            //
//                                                                      //
// $Revision: 1.8 $ $Date: 2006/06/29 08:11:58 $ $Author: boeheim $
// ******************************************************************** //

using namespace std;

#include <arpa/inet.h>
#include <errno.h>

#include "autoconfig.h"
#include "psstream.h"
#include "sockettcp.h"

extern int errno;

// ====================================================================
// Create the underlying OS socket for the object.
// ====================================================================

void SocketTCP::Create(void)
{
   int trueval = 1;

   if (fd >= 0) return;         // Already created.

   fd = socket(AF_INET, SOCK_STREAM, 0);
   
   if (fd < 0) throw IOErr("socket()", errno, filename);
   
   if (setsockopt(fd, SOL_SOCKET, SO_REUSEADDR, (char *)&trueval, sizeof(trueval)) < 0)
      throw IOErr("setsockopt()", errno, filename);
}

// ====================================================================
// Virtual destructor.
// ====================================================================

SocketTCP::~SocketTCP(void)
{

}

// ====================================================================
// Initialize a socket as a listen socket on a specified service or port
// Used for the server side of a connection.
// ====================================================================

void SocketTCP::Listen(void)
{
   if (fd < 0) Create();

   if (listen(fd, SOMAXCONN) < 0)
      throw IOErr("listen()", errno, filename);
}

// ====================================================================
// Accept a connection on a listen socket and initialize a new socket for
// the connection.  Return a new socket object for the connected socket.
// ====================================================================

SocketTCP *SocketTCP::Accept(void)
{
   SocketTCP *newsocket = new SocketTCP;
   
   try
   {
      Accept(newsocket);
   }
   catch (IOErr e)
   {
      delete newsocket;
      throw e;
   }
   return newsocket;
}


SocketTCP *SocketTCP::Accept(SocketTCP *newsocket)
{
   Host from;
   socklen_t fromlen = from.Len();
   int newfd;
   struct linger lopt;

   newfd = ::accept(fd, from.Ptr(), &fromlen);

   if (newfd < 0) throw IOErr("accept()", errno, filename);

   newsocket->SetFD(newfd);
   SmallBuffer temp;
   from.GetPeerName();
   temp += from.Name();
   temp += ":";
   temp += from.Port();
   newsocket->filename = (char *) temp.Cstr();

   lopt.l_onoff = 1;
   lopt.l_linger = 100;
   
   if (setsockopt(newfd, SOL_SOCKET, SO_LINGER, (char *)&lopt, sizeof(lopt)) < 0)
      throw IOErr("setsockopt()", errno, newsocket->filename);

   return newsocket;
}

