// ====================================================================
// terminal.cc - Implementation of the Terminal class
// ====================================================================

// ******************************************************************** //
//                  SLAC Console Management Tools                       //
//         Chuck Boeheim, Stanford Linear Accelerator Center            //
//                                                                      //
// $Revision: 1.5 $ $Date: 2005/02/09 00:48:13 $ $Author: boeheim $
// ******************************************************************** //

using namespace std;

#include "global.h"
#include "consolec.h"
#include "terminal.h"
#include "syslog.h"

#include <iostream>

// ====================================================================
// SendPassword() - read a password from the terminal and send it
//                  to the server.
// ====================================================================

string Terminal::GetLine(void)
{
   char c;
   string line;
  
   for (;;)
   {
      Read (&c, 1);             // Read one character at a time.
      if (c == '\n') break;     //    Until a newline.
      line += c;                //   Append to collected line.
   };        

   return line;
}

// ====================================================================
// ReaderProc() - loops reading from the terminal and
// writing the data to the the socket connection 
// ====================================================================   


void Terminal::ReaderProc (void)
{
   for(;;)
   {
      buffer.Clear();           // Start with a clean buffer.
      
      try
      {
         Read(buffer);          // Read from the terminal

         if (buffer.Size() == 0) break;

         server->Send(buffer); // and write it to the socket connection
      }

      catch (IOErr e)           // Handle any errors.
      {
         SYSLOG << Severity(Syslog::error) 
                << e.routine 
                << ": " << e.filename 
                << ": " << e.message 
                << endl;
         
         return;
      }
   }

   if (GLOBAL.Verbose()) cerr << "Terminal reader thread ended\r\n";
}



    
