// ====================================================================
// terminal.hh - A class to represent the physical terminal attached
//                to the client program.  It has a pointer to the 
//                server connection and a reader thread.
// ====================================================================

// ******************************************************************** //
//                  SLAC Console Management Tools                       //
//         Chuck Boeheim, Stanford Linear Accelerator Center            //
//                                                                      //
// $Revision: 1.1 $ $Date: 2005/02/09 00:38:51 $ $Author: boeheim $
// ******************************************************************** //

#ifndef terminal_h
#define terminal_h

#include "server.h"

#include "ttyfile.h"
#include "reader.h"
#include "buffer.h"
#include "defaults.h"

#include <termios.h>

class Server;

class Terminal : public TTYFile, public Reader
{
private:
   
   Server *server;
   Buffer buffer;
   struct termios state;

public:
   
   Terminal(void) { fd = 0; };
   ~Terminal(void) {};

   void SaveState(void) { TCGetAttr(state); };
   void RestoreState(void) { TCSetAttr(TCSANOW, state); };
   
   // Server method initializes server pointer.

   void RegisterServer(Server *serv) { server = serv; }; 
   string GetLine(void);
   void ReaderProc(void);
   
};

#endif
