// ====================================================================
// Class for encapsulating tty files and their actions.
// ttys have the basic operation of files plus a number
// of specific hardware and protocol options.
// ====================================================================

// ******************************************************************** //
//                  SLAC Console Management Tools                       //
//         Chuck Boeheim, Stanford Linear Accelerator Center            //
//                                                                      //
// $Revision: 1.2 $ $Date: 2006/06/23 15:59:27 $ $Author: boeheim $
// ******************************************************************** //

#ifndef ttyfile_hh
#define ttyfile_hh

#include "file.h"

#include <fcntl.h>
#include <sys/ioctl.h>
#include <vector>

class TTYFile : public File
{
   int baud;                    // the baud on this console port        
   int setparity;               // the parity on this line
   int clrparity;
   bool readonly;               // Not if tty not writable.
   bool autobaud;               // Try all baud settings.

public:
   TTYFile(void);
   TTYFile(int newfd) : File(newfd) { TTYFile(); };
   
   void   SetMode(const string &mode);
   string GetSpeed(void);
   void   SetSpeed(string);
   string NextSpeed(void);
   bool Autobaud(void) { return autobaud; };
   void TCGetAttr(struct termios &sbuf);
   void TCSetAttr(int flags, struct termios &sbuf);
   int  TCGetPgrp(void);
   void TCSetPgrp(int grp);
   void SetControlTTY(void);
   void NoControlTTY(void);
   void SendBreak(void);
   int  FlowToggle(void);
   void FlowOn(void);
   void FlowOff(void);
   void Hangup(void);
   void TabsToggle(void);
   int  SetSane(void);
   int  SetRaw(void);
   bool ReadOnly(void) { return readonly; };

   void SetRTS(bool);
   void SetCTS(bool);
   void SetDSR(bool);
   void SetCAR(bool);
   void SetDTR(bool);
   void SetRI (bool);
   
   enum { SigRTS=0, SigCTS, SigDSR, SigCAR, SigDTR, SigRI };
   vector<bool> GetSignals(void);
   
};
   

#endif
